#include "conf.h"

int cfg_changed = 0;

int cfg_use_colors = 1;
int cfg_draw_interval = 1.0f;
char cfg_fill_char = '*';
char cfg_bg_char = '.';
int cfg_hide_dummy_interfaces = 0;

static void p_conf_parse(char *);
static void p_conf_parse_layout(char *);
static void p_conf_parse_option(char *id, char *value);
static int p_conf_parse_color(const char *);


#ifdef HAVE_USE_DEFAULT_COLORS
layout_t layout[] =
{
    {-1, -1, 0},                           /* dummy, not used */
    {-1, -1, 0},                           /* default */
    {-1, -1, A_REVERSE},                   /* statusbar */
    {-1, -1, 0},                           /* header */
    {-1, -1, 0},                           /* list */
    {-1, -1, A_REVERSE}                    /* selected */
};
#else
layout_t layout[] =
{
    {0, 0, 0},                              /* dummy, not used */
    {COLOR_BLACK, COLOR_WHITE, 0},          /* default */
    {COLOR_BLACK, COLOR_WHITE, A_REVERSE},  /* statusbar */
    {COLOR_BLACK, COLOR_WHITE, 0},          /* header */
    {COLOR_BLACK, COLOR_WHITE, 0},          /* list */
    {COLOR_BLACK, COLOR_WHITE, A_REVERSE}   /* selected */
};
#endif

void
conf_read(const char *path)
{
    FILE *f;
    char buf[1024];

    if (!(f = fopen(path, "r")))
        return;

    memset(buf, 0, sizeof(buf));

    while (fgets(buf, sizeof(buf), f))
    {
        char *p;

        if ((p = strchr(buf, '\r')))
            *p = '\0';

        if ((p = strchr(buf, '\n')))
            *p = '\0';

        if (*buf)
            p_conf_parse(buf);
    }

    fclose(f);
}

#define SPLIT(str, var) {                    \
    char *s;                                 \
    for (s = str; *s; s++) {                 \
        if (*s == ' ' || *s == '\t')         \
            break;                           \
    }                                        \
    if (!*s)                                 \
        return;                              \
    *s = '\0';                               \
    for (++s;*s == ' ' || *s == '\t'; s++);  \
    if (!*s)                                 \
        return;                              \
    var = s;                                 \
}



static void
p_conf_parse(char *buf)
{
    char *tok;

    if (*buf == '#')
        return;

    SPLIT(buf, tok)

    if (!strcasecmp(buf, "layout"))
        p_conf_parse_layout(tok);
    else
        p_conf_parse_option(buf, tok);
}


static void
p_conf_parse_layout(char *buf)
{
    char *id, *fg, *bg, *attr, *s;
    layout_t l;

    id = buf;
    SPLIT(buf, fg)
    SPLIT(fg, bg)

    for (s = bg; *s; s++)
    {
        if (*s == ' ' || *s == '\t')
            break;
    }

    if (*s)
    {
        *s = '\0';
        s++;

        for (; *s == ' ' || *s == '\t'; s++);
        if (*s)
            attr = s;
        else
            attr = NULL;
    }
    else
        attr = NULL;

    if ((l.fg = p_conf_parse_color(fg)) == 31337)
        return;

    if ((l.bg = p_conf_parse_color(bg)) == 31337)
        return;

    l.attr = 0;

    if (attr)
    {
        if (!strcasecmp(attr, "reverse"))
            l.attr |= A_REVERSE;
        else if (!strcasecmp(attr, "bold"))
            l.attr |= A_BOLD;
        else if (!strcasecmp(attr, "underline"))
            l.attr |= A_UNDERLINE;
    }

#define COPY(id) {                 \
    layout[id].fg = l.fg;          \
    layout[id].bg = l.bg;          \
    layout[id].attr = l.attr;      \
}

    if (!strcasecmp(id, "default"))
        COPY(LAYOUT_DEFAULT)
    else if (!strcasecmp(id, "statusbar"))
        COPY(LAYOUT_STATUSBAR)
    else if (!strcasecmp(id, "header"))
        COPY(LAYOUT_HEADER)
    else if (!strcasecmp(id, "list"))
        COPY(LAYOUT_LIST)
    else if (!strcasecmp(id, "selected"))
        COPY(LAYOUT_SELECTED)
}

static int
p_conf_parse_color(const char *color)
{
    if (!strcmp(color, "default"))
        return -1;
    else if (!strcasecmp(color, "black"))
        return COLOR_BLACK;
    else if (!strcasecmp(color, "red"))
        return COLOR_RED;
    else if (!strcasecmp(color, "green"))
        return COLOR_GREEN;
    else if (!strcasecmp(color, "yellow"))
        return COLOR_YELLOW;
    else if (!strcasecmp(color, "blue"))
        return COLOR_BLUE;
    else if (!strcasecmp(color, "magenta"))
        return COLOR_MAGENTA;
    else if (!strcasecmp(color, "cyan"))
        return COLOR_CYAN;
    else if (!strcasecmp(color, "white"))
        return COLOR_WHITE;
    else
        return 31337;
}


static void
p_conf_parse_option(char *id, char *value)
{

#define SET(var, flag) { if (!(cfg_changed & flag)) var = (!strcasecmp(value, "on")) ? 1 : 0; }
#define SET_INT(var, flag) { if (!(cfg_changed & flag)) var = atoi(value); }
#define SET_CHAR(var, flag) { if (!(cfg_changed & flag)) var = value[0]; }

    if (!strcasecmp(id, "colors"))
        SET(cfg_use_colors, CFG_USE_COLORS)
    else if (!strcasecmp(id, "drawinterval"))
        SET_INT(cfg_draw_interval, CFG_DRAW_INTERVAL)
    else if (!strcasecmp(id, "hidedummyinterfaces"))
        SET(cfg_hide_dummy_interfaces, CFG_HIDE_DUMMY_INTERFACES)
    else if (!strcasecmp(id, "fillchar"))
        SET_CHAR(cfg_fill_char, CFG_FILL_CHAR)
    else if (!strcasecmp(id, "bgchar"))
        SET_CHAR(cfg_bg_char, CFG_BG_CHAR)

#undef SET_INT
#undef SET
}
