/*
 * draw.c
 *
 * Copyright (C) 2002 Thomas Graf <tgr@kaosu.ch>
 */

#include "draw.h"
#include "intf.h"
#include "conf.h"

/*
 * must be implemented by the application using this module
 * is called by this module whenever a fatal error occurs
 */
extern void quit (const char *fmt, ...);

extern intf_t  intf[MAX_IF];
extern layout_t layout[];
extern int cfg_use_colors;

#ifdef HAVE_CURSES

static int     bdev                    = 0;
int row                                = 0;
static int rows, cols;
extern WINDOW *win;


/*
 * Safe move(++row,0)
 */
#define NEXT_ROW {                      \
    row++;                              \
    if (row >= rows-1)                  \
        return;                         \
    move(row,0);                        \
}
#endif


static double p_sumup(intf_cnt_t l, char **unit);
static void p_intf_draw_ascii(int inum);
static void p_prepare_graph(int i, char **unit_x, char **rx_unit_y,
        char **tx_unit_y,
        int *index, float *rx_scale, float *tx_scale, char **rx_start,
        char **tx_start);

enum bandwidth m_bandwidth = overview;
enum graphic m_b_graphic = minute;
enum unit m_unit = dyanmic;

#ifdef HAVE_CURSES
static void p_print_full_line(const char *fmt, ...);
static void p_intf_print_graphic(void);
static void p_intf_print_bandwidth(void);
static void p_intf_print_detailed(void);



void
intf_draw(void)
{
    if (m_bandwidth == graphic || m_bandwidth == detailed)
        if (!intf[bdev].name[0])
            return;

    row = 0;
    move(0,0);

    getmaxyx(win, rows, cols);

    if (cols < 80)
    {
        p_print_full_line("Screen is not wide enough");
        refresh();
        return;
    }

    if (cfg_use_colors)
        attrset(COLOR_PAIR(LAYOUT_STATUSBAR) | layout[LAYOUT_STATUSBAR].attr);
    else
        attrset(A_REVERSE);

    switch (m_bandwidth)
    {
        case overview:
            p_print_full_line(" Bandwidth Overview");
            break;

        case detailed:
            p_print_full_line(" Detailed statistics for %s", intf[bdev].name);
            break;

        case graphic:
            p_print_full_line(" Graphical statistics for %s", intf[bdev].name);
            break;
    }

    if (cfg_use_colors)
        attrset(COLOR_PAIR(LAYOUT_DEFAULT) | layout[LAYOUT_DEFAULT].attr);
    else
        attroff(A_REVERSE);

    NEXT_ROW;
    p_print_full_line("");

    switch (m_bandwidth)
    {
        case overview:
            p_intf_print_bandwidth();
            break;

        case detailed:
            p_intf_print_detailed();
            break;

        case graphic:
            p_intf_print_graphic();
            break;
    }

    for (; row < rows-2;)
    {
        move(++row, 0);
        p_print_full_line("");
    }

    row = rows-1;
    move(row, 0);

    if (cfg_use_colors)
        attrset(COLOR_PAIR(LAYOUT_STATUSBAR) | layout[LAYOUT_STATUSBAR].attr);
    else
        attrset(A_REVERSE);

    if (m_bandwidth == graphic)
        p_print_full_line(" Mode: (o)verview (g)raphical (d)etailed        Time Scale:(s)ec/(m)in/(h)our");
    else
        p_print_full_line(" Mode: (o)verview (g)raphical (d)etailed");

    attrset(0);

    refresh();
}


static void
p_intf_print_bandwidth(void)
{
    int i;


    if (cfg_use_colors)
        attrset(COLOR_PAIR(LAYOUT_HEADER) | layout[LAYOUT_HEADER].attr);

    NEXT_ROW;
    p_print_full_line("#   Interface          RX Rate       RX #    TX Rate       TX #");

    NEXT_ROW;
    hline(ACS_HLINE, cols);

    if (cfg_use_colors)
        attrset(COLOR_PAIR(LAYOUT_LIST) | layout[LAYOUT_LIST].attr);

    for (i=0; i < MAX_IF; i++)
    {
        if (intf[i].name[0])
        {
            if (row >= rows-1)
                return;

            NEXT_ROW;

            if (i == bdev)
            {
                if (cfg_use_colors)
                    attrset(COLOR_PAIR(LAYOUT_SELECTED) | layout[LAYOUT_SELECTED].attr);
                else
                    attron(A_REVERSE);
            }

            printw("%-3d ", i);

            if (i == bdev)
            {
                if (cfg_use_colors)
                    attrset(COLOR_PAIR(LAYOUT_LIST) | layout[LAYOUT_LIST].attr);
                else
                    attroff(A_REVERSE);
            }

            {
                float rx_u, tx_u;
                char *rx_un, *tx_un;
                
                rx_u = p_sumup(intf[i].rx_bytes.rate, &rx_un);
                tx_u = p_sumup(intf[i].tx_bytes.rate, &tx_un);

                p_print_full_line("%-14s %10.2f%s %10.1f %10.2f%s %10.1f",
                    intf[i].name, rx_u, rx_un, (float) intf[i].rx_packets.rate,
                    tx_u, tx_un, (float) intf[i].tx_packets.rate);
            }
        }
    }

    if (cfg_use_colors)
        attrset(COLOR_PAIR(LAYOUT_DEFAULT) | layout[LAYOUT_DEFAULT].attr);

    if ((rows - row) >= 23 && cols >= 80)
    {
        NEXT_ROW;
        p_print_full_line("");
        p_intf_print_graphic();
    }
}

static void
p_intf_print_graphic(void)
{
    int i, w, gi;
    char *unit_x, *rx_unit_y, *tx_unit_y, *rx_g, *tx_g;
    float rx_scale[6], tx_scale[6];

    if (rows < 20)
    {
        NEXT_ROW;
        p_print_full_line("Need 19 lines high screen");
        return;
    }
    

    i = bdev;

    if (intf[i].name[0]) {

        p_prepare_graph(i, &unit_x, &rx_unit_y, &tx_unit_y, &gi, rx_scale,
                tx_scale, &rx_g, &tx_g);

        NEXT_ROW;
        p_print_full_line("       %s", rx_unit_y);

        NEXT_ROW;
        p_print_full_line("");

        for (w=5; w >= 0; w--)
        {
            if (w == 5)
                p_print_full_line("RX");
            else
                p_print_full_line("");

            move(row, 2);

            printw("%8.2f ", rx_scale[w]);
            addstr(rx_g + (w * 61) );

            NEXT_ROW;
        }

        p_print_full_line("           1   5   10   15   20   25   30   35   " \
                "40   45   50   55   60 %s",
                unit_x);
        NEXT_ROW;
        p_print_full_line("");

        NEXT_ROW;
        p_print_full_line("       %s", tx_unit_y);

        NEXT_ROW;
        p_print_full_line("");

        for (w=5; w >= 0; w--)
        {
            if (w == 5)
                p_print_full_line("TX");
            else
                p_print_full_line("");

            move(row, 2);

            printw("%8.2f ", tx_scale[w]);
            addstr(tx_g + (w * 61) );

            NEXT_ROW;
        }

        p_print_full_line("           1   5   10   15   20   25   30   35   " \
            "40   45   50   55   60 %s", unit_x);

    }
}

static void
p_intf_print_detailed(void)
{
    int i;

    i = bdev;

    if (intf[i].name[0])
    {
        float rx_u, tx_u;
        char *rx_un, *tx_un;

        NEXT_ROW;
        p_print_full_line("                          RX               TX");

        NEXT_ROW;

        rx_u = p_sumup(intf[i].rx_bytes.cnt, &rx_un);
        tx_u = p_sumup(intf[i].tx_bytes.cnt, &tx_un);

        p_print_full_line(" Bytes:         %12.2f %s  %12.2f %s", rx_u, rx_un, tx_u, tx_un);

        NEXT_ROW;
        p_print_full_line(" Packets:       %12llu     %12llu", 
                intf[i].rx_packets.cnt, intf[i].tx_packets.cnt);
        NEXT_ROW;
        p_print_full_line(" Errors:        %12llu     %12llu",
                intf[i].rx_errors, intf[i].tx_errors);
        NEXT_ROW;
        p_print_full_line(" Drop:          %12llu     %12llu",
                intf[i].rx_drop, intf[i].tx_drop);
        NEXT_ROW;
        p_print_full_line(" Fifo:          %12llu     %12llu",
                intf[i].rx_fifo, intf[i].tx_fifo);
        NEXT_ROW;
        p_print_full_line(" Frame:         %12llu     %12llu",
                intf[i].rx_frame, intf[i].tx_frame);
        NEXT_ROW;
        p_print_full_line(" Compressed:    %12llu     %12llu",
                intf[i].rx_compressed, intf[i].tx_compressed);
        NEXT_ROW;
        p_print_full_line(" Multicast:     %12llu     %12llu",
                intf[i].rx_multicast, intf[i].tx_multicast);
#if defined SYS_SUNOS && defined HAVE_KSTAT
        NEXT_ROW;
        p_print_full_line(" Broadcast:     %12llu     %12llu",
                intf[i].rx_broadcast, intf[i].tx_broadcast);
#endif
    }
}

int
intf_handle_input(int ch)
{
    int w;

    switch (ch) 
    {
        case 'o':
        case 0x1b: /* ESC */
            clear();
            m_bandwidth = overview;
            return 1;

        case 'd':
            clear();
            m_bandwidth = detailed;
            return 1;

        case 's':
            clear();
            m_b_graphic = minute;
            return 1;

        case 'm':
            clear();
            m_b_graphic = hour;
            return 1;

        case 'h':
            clear();
            m_b_graphic = day;
            return 1;

        case 'g':
        case 0x0d: /* CR */
            clear();
            m_bandwidth = graphic;
            return 1;

        case KEY_DOWN:
            if (bdev+1 == MAX_IF-1)
                break;

            for (w=bdev+1; !intf[w].name[0] && w < MAX_IF; w++);

            if (w < MAX_IF && intf[w].name[0])
            {
                bdev = w;
                return 1;
            }
            break;

        case KEY_UP:
            if (bdev-1 < 0)
                break;

            for (w=bdev-1; !intf[w].name[0] && w >= 0; w--);

            if (w >= 0 && intf[w].name[0])
            {
                bdev = w;
                return 1;
            }
            break;

        case KEY_LEFT:
            if (m_b_graphic == hour)
            {
                m_b_graphic = minute;
                return 1;
            }
            else if (m_b_graphic == day)
            {
                m_b_graphic = hour;
                return 1;
            }
            break;

        case KEY_RIGHT:
            if (m_b_graphic == hour)
            {
                m_b_graphic = day;
                return 1;
            }
            else if (m_b_graphic == minute)
            {
                m_b_graphic = hour;
                return 1;
            }
            break;

        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
            if ((ch - 48) >= 0 && (ch - 48) <= 9)
            {
                if (intf[ch - 48].name[0])
                {
                    bdev = ch - 48;
                    return 1;
                }
            }
            break;
    }

    return 0;
}
#endif



void
intf_draw_ascii(void)
{
    int i;

    switch (m_bandwidth)
    {
        case overview:
            printf("Interface          RX Rate       RX #    TX Rate        " \
                    "TX #\n");
            break;

        case detailed:
            break;

        case graphic:

            break;
    }

    for (i=0; i < MAX_IF; i++)
        if (intf[i].name[0])
            p_intf_draw_ascii(i);
}

void
intf_draw_html(const char *path)
{
    char ofile[FILENAME_MAX];
    FILE *f;
    int i;

    memset(ofile, 0, sizeof(ofile));
    strncpy(ofile, path, sizeof(ofile)-1);
    strncat(ofile, "/index.html", sizeof(ofile)-strlen(ofile)-1);

    if (!(f = fopen(ofile, "w")))
        quit("fopen(%s) failed: %s\n", ofile, strerror(errno));

    fprintf(f,
        "<!DOCTYPE html PUBLIC \"-//W3c/DTD XHTML 1.0 Strict//EN\"\n"  \
        "    \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n" \
        "<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"en\"\n"   \
        "       xml:lang=\"en\">\n"                                    \
        "<head>\n"                                                     \
        "<title>Interface Overview</title>\n"                          \
        "</head>\n"                                                    \
        "<body>\n"                                                     \
        "<h1>Interface Overview</h1>\n"                                \
        "<table>\n"                                       \
        "<tr>\n"                                                       \
        "<td><b>Name</b></td>\n"                                       \
        "<td><b>RX Rate</b></td><td><b>RX #</b></td>\n"                \
        "<td><b>TX Rate</b></td><td><b>TX #</b></td>\n"                \
        "<td><b>graph</b></td><td><b>details</b></td></tr>\n");

    for (i=0; i < MAX_IF; i++)
    {
        char *unit;

        if (!intf[i].name[0])
            continue;

        fprintf(f,
            "<tr>\n"                                                   \
            "<td><b>%s</b></td>"                                       \
            "<td>%10.2f",
            intf[i].name, p_sumup(intf[i].rx_bytes.rate, &unit));
        fprintf(f,
            "%s</td>\n"                                                \
            "<td>%10.1f</td>\n"                                        \
            "<td>%10.2f",
            unit, (float) intf[i].rx_packets.rate,
            p_sumup(intf[i].tx_bytes.rate, &unit));

        fprintf(f,
            "%s</td>\n" \
            "<td>%10.1f</td>\n" \
            "<td><a href=\"%s.s.graph.html\">graph</a></td>\n" \
            "<td><a href=\"%s.details.html\">details</a></td>\n" \
            "</tr>\n", unit, (float) intf[i].tx_packets.rate,
            intf[i].name, intf[i].name);

    }

    fprintf(f,
        "</table>\n"                                                   \
        "</body>\n"                                                    \
        "</html>\n");

    fclose(f);

    for (i=0; i < MAX_IF; i++)
    {
        int w, index, p;
        char *unit, *unit_x, *rx_unit_y, *tx_unit_y, *rx_g, *tx_g;
        float rx_scale[6], tx_scale[6];

        if (!intf[i].name[0])
            continue;

        m_b_graphic = minute;

        for (p = 0; p < 3; p++)
        {
            switch (m_b_graphic)
            {
                case minute:
                    snprintf(ofile, sizeof(ofile), "%s/%s.s.graph.html",
                            path, intf[i].name);
                    break;

                case hour:
                    snprintf(ofile, sizeof(ofile), "%s/%s.m.graph.html",
                            path, intf[i].name);
                    break;

                case day:
                    snprintf(ofile, sizeof(ofile), "%s/%s.h.graph.html",
                            path, intf[i].name);
                    break;
            }


            if (!(f = fopen(ofile, "w")))
                quit("fopen(%s) failed: %s\n", ofile, strerror(errno));

            fprintf(f,
                "<!DOCTYPE html PUBLIC \"-//W3c/DTD XHTML 1.0 Strict//EN\"\n"  \
                "    \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n" \
                "<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"en\"\n"   \
                "       xml:lang=\"en\">\n"                                    \
                "<head>\n"                                                     \
                "<title>Graph for %s</title>\n"                                \
                "</head>\n"                                                    \
                "<body>\n"                                                     \
                "<h1>Graph for %s</h1>\n"                                      \
                "<pre>\n", intf[i].name, intf[i].name);

            p_prepare_graph(i, &unit_x, &rx_unit_y, &tx_unit_y, &index,
                    rx_scale, tx_scale, &rx_g, &tx_g);

            for (w=5; w >= 0; w--)
            {
                if (w == 5)
                    fprintf(f, "RX    %s\n", rx_unit_y);

                fprintf(f, "%8.2f ", rx_scale[w]);
                fprintf(f, "%s\n", (rx_g + (w * 61)));
            }
            fprintf(f, "         1   5   10   15   20   25   30   35   40   " \
                    "45   50   55   60 %s\n", unit_x);

            for (w=5; w >= 0; w--)
            {
                if (w == 5)
                    fprintf(f, "TX    %s\n", tx_unit_y);

                fprintf(f, "%8.2f ", tx_scale[w]);
                fprintf(f, "%s\n", (tx_g + (w * 61)));
            }
            fprintf(f, "         1   5   10   15   20   25   30   35   40   " \
                    "45   50   55   60 %s\n\n", unit_x);
            
            fprintf(f,
                "</pre>\n"                                                     \
                "<p>Unit: <a href=\"%s.s.graph.html\">Second</a>,\n" \
                "<a href=\"%s.m.graph.html\">Minute</a>,\n" \
                "<a href=\"%s.h.graph.html\">Hour</a></p>\n" \
                "<p><a href=\"index.html\">Back to interface overview</a></p>" \
                "</body>\n"                                        \
                "</html>\n", intf[i].name, intf[i].name, intf[i].name);
            

            fclose(f);

            switch (m_b_graphic)
            {
                case minute:
                    m_b_graphic = hour;
                    break;

                case hour:
                    m_b_graphic = day;
                    break;

                case day:
                    m_b_graphic = minute;
                    break;
            }
        }

        snprintf(ofile, sizeof(ofile), "%s/%s.details.html",
                path, intf[i].name);

        if (!(f = fopen(ofile, "w")))
            quit("fopen(%s) failed: %s\n", ofile, strerror(errno));

        fprintf(f,
            "<!DOCTYPE html PUBLIC \"-//W3c/DTD XHTML 1.0 Strict//EN\"\n"  \
            "    \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n" \
            "<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"en\"\n"   \
            "       xml:lang=\"en\">\n"                                    \
            "<head>\n"                                                     \
            "<title>Details for %s</title>\n"                              \
            "</head>\n"                                                    \
            "<body>\n"                                                     \
            "<h1>Details for %s</h1>\n"                                    \
            "<table>\n", intf[i].name, intf[i].name);

        fprintf(f,
            "<tr>\n"                                                       \
            "<td><b>Bytes</b></td>\n"                                      \
            "<td>%12.2f\n", p_sumup(intf[i].rx_bytes.cnt, &unit));

        fprintf(f,
            "%s</td>\n"                                                    \
            "<td>%12.2f", unit, p_sumup(intf[i].tx_bytes.cnt, &unit));
        fprintf(f,
            "%s</td>\n" \
            "</tr>\n", unit);

        fprintf(f,
            "<tr>\n" \
            "<td><b>Packets</b></td>\n" \
            "<td>%12llu</td>\n" \
            "<td>%12llu</td>\n" \
            "</tr>\n", intf[i].rx_packets.cnt, intf[i].tx_packets.cnt);

        fprintf(f,
            "<tr>\n" \
            "<td><b>Errors</b></td>\n" \
            "<td>%12llu</td>\n" \
            "<td>%12llu</td>\n" \
            "</tr>\n", intf[i].rx_errors, intf[i].tx_errors);

        fprintf(f,
            "<tr>\n" \
            "<td><b>Drop</b></td>\n" \
            "<td>%12llu</td>\n" \
            "<td>%12llu</td>\n" \
            "</tr>\n", intf[i].rx_drop, intf[i].tx_drop);

        fprintf(f,
            "<tr>\n" \
            "<td><b>Fifo</b></td>\n" \
            "<td>%12llu</td>\n" \
            "<td>%12llu</td>\n" \
            "</tr>\n", intf[i].rx_fifo, intf[i].tx_fifo);

        fprintf(f,
            "<tr>\n" \
            "<td><b>Frame</b></td>\n" \
            "<td>%12llu</td>\n" \
            "<td>%12llu</td>\n" \
            "</tr>\n", intf[i].rx_frame, intf[i].tx_frame);

        fprintf(f,
            "<tr>\n" \
            "<td><b>Compressed</b></td>\n" \
            "<td>%12llu</td>\n" \
            "<td>%12llu</td>\n" \
            "</tr>\n", intf[i].rx_compressed, intf[i].tx_compressed);
        
        fprintf(f,
            "<tr>\n" \
            "<td><b>Multicast</b></td>\n" \
            "<td>%12llu</td>\n" \
            "<td>%12llu</td>\n" \
            "</tr>\n", intf[i].rx_multicast, intf[i].tx_multicast);
        
#if defined SYS_SUNOS && defined HAVE_KSTAT
        fprintf(f,
            "<tr>\n" \
            "<td><b>Broadcast</b></td>\n" \
            "<td>%12llu</td>\n" \
            "<td>%12llu</td>\n" \
            "</tr>\n", intf[i].rx_broadcast, intf[i].tx_broadcast);
#endif

        fprintf(f,
            "</table>\n"                                                   \
            "<p><a href=\"index.html\">Back to interface overview</a></p>\n" \
            "</body>\n"                                                    \
            "</html>\n");

        fclose(f);

    }
}


static void
p_intf_draw_ascii(int i)
{
    char *unit;
    int w, index;
    char *unit_x, *rx_unit_y, *tx_unit_y, *rx_g, *tx_g;
    float rx_scale[6], tx_scale[6];

    switch (m_bandwidth)
    {
        case overview:
            printf("%-14s %10.2f", intf[i].name, p_sumup(intf[i].rx_bytes.rate,
                        &unit));
            printf("%s", unit);
            printf("%10.1f", (float) intf[i].rx_packets.rate);
            printf("%10.2f", p_sumup(intf[i].tx_bytes.rate, &unit));
            printf("%s", unit);
            printf("%10.1f\n", (float) intf[i].tx_packets.rate);
            break;

        case detailed:
            printf("Interface: %s\n", intf[i].name);
            printf("  Bytes:         %12.2f", p_sumup(intf[i].rx_bytes.cnt,
                        &unit));
            printf(" %s", unit);
            printf("  %12.2f", p_sumup(intf[i].tx_bytes.cnt, &unit));
            printf(" %s\n", unit);
            printf("  Packets:       %12llu     %12llu\n",
                    intf[i].rx_packets.cnt, intf[i].tx_packets.cnt);
            printf("  Errors:        %12llu     %12llu\n",
                    intf[i].rx_errors, intf[i].tx_errors);
            printf("  Drop:          %12llu     %12llu\n",
                    intf[i].rx_drop, intf[i].tx_drop);
            printf("  Fifo:          %12llu     %12llu\n",
                    intf[i].rx_fifo, intf[i].tx_fifo);
            printf("  Frame:         %12llu     %12llu\n",
                    intf[i].rx_frame, intf[i].tx_frame);
            printf("  Compressed:    %12llu     %12llu\n",
                    intf[i].rx_compressed, intf[i].tx_compressed);
            printf("  Multicast:     %12llu     %12llu\n",
                    intf[i].rx_multicast, intf[i].tx_multicast);
#if defined SYS_SUNOS && defined HAVE_KSTAT
            printf("  Broadcast:     %12llu     %12llu\n",
                    intf[i].rx_broadcast, intf[i].tx_broadcast);
#endif
            printf("\n");
            break;

        case graphic:

            p_prepare_graph(i, &unit_x, &rx_unit_y, &tx_unit_y, &index,
                    rx_scale, tx_scale, &rx_g, &tx_g);

            printf("Interface: %s\n", intf[i].name);

            for (w=5; w >= 0; w--)
            {
                if (w == 5)
                    printf("RX    %s\n", rx_unit_y);

                printf("%8.2f ", rx_scale[w]);
                printf("%s\n", (rx_g + (w * 61)));
            }
            printf("         1   5   10   15   20   25   30   35   40   " \
                    "45   50   55   60 %s\n", unit_x);

            for (w=5; w >= 0; w--)
            {
                if (w == 5)
                    printf("TX    %s\n", tx_unit_y);

                printf("%8.2f ", tx_scale[w]);
                printf("%s\n", (tx_g + (w * 61)));
            }
            printf("         1   5   10   15   20   25   30   35   40   " \
                    "45   50   55   60 %s\n\n", unit_x);
            break;
    }
}


static double
p_sumup(intf_cnt_t l, char **unit)
{
    *unit = "B ";

    switch (m_unit)
    {
        case byte:
            *unit = "B ";
            return (double) l;

        case kilobyte:
            *unit = "KB";
            return ((double) l / 1024);
            
        case megabyte:
            *unit = "MB";
            return ((double) l / 1048576);

        case gigabyte:
            *unit = "GB";
            return ((double) l / 1073741824);

        case terabyte:
            *unit = "TB";
            return ((double) l /1099511627776);

        case dyanmic:
            if (l >= 1099511627776)
            {
                *unit = "TB";
                return ((double) l) / 1099511627776;
            }
            else if (l >= 1073741824)
            {
                *unit = "GB";
                return ((double) l) / 1073741824;
            }
            else if (l >= 1048576)
            {
                *unit = "MB";
                return ((double) l) / 1048576;
            }
            else if (l >= 1024)
            {
                *unit = "KB";
                return ((double) l) / 1024;
            }
            else
            {
                *unit = "B ";
                return (double) l;
            }
            break;
    }

    return (double) l;
}

#ifdef HAVE_CURSES
static void
p_print_full_line(const char *fmt, ...)
{
    va_list args;
    char buf[1024];
    int x, y;

    getyx(stdscr, y, x);

    memset(buf, 0, sizeof(buf));

    va_start(args, fmt);
    vsnprintf(buf, sizeof(buf), fmt, args);
    va_end(args);

    if (strlen(buf) > cols-x)
        buf[cols-x] = '\0';
    else
        memset(&buf[strlen(buf)], ' ', cols-strlen(buf)-x);

    addstr(buf);
}


#endif

static void
p_prepare_graph(
        int i,
        char **unit_x,
        char **rx_unit_y,
        char **tx_unit_y,
        int *index,
        float *rx_scale,
        float *tx_scale,
        char **rx_start,
        char **tx_start)
{
    int n, w;

    switch (m_b_graphic)
    {
        case minute:
            *index = intf[i].s.index;
            *unit_x = "s";

            for (n = 0; n < 6; n++)
            {
                rx_scale[n] = intf[i].s.rx_scale[n];
                tx_scale[n] = intf[i].s.tx_scale[n];
            }

            *rx_start = &intf[i].s.rx_graph[0][0];
            *tx_start = &intf[i].s.tx_graph[0][0];
            break;

        case hour:
            *index = intf[i].m.index;
            *unit_x = "m";

            for (n = 0; n < 6; n++)
            {
                rx_scale[n] = intf[i].m.rx_scale[n];
                tx_scale[n] = intf[i].m.tx_scale[n];
            }

            *rx_start = &intf[i].m.rx_graph[0][0];
            *tx_start = &intf[i].m.tx_graph[0][0];
            break;

        case day:
            *index = intf[i].h.index;
            *unit_x = "h";

            for (n = 0; n < 6; n++)
            {
                rx_scale[n] = intf[i].h.rx_scale[n];
                tx_scale[n] = intf[i].h.tx_scale[n];
            }

            *rx_start = &intf[i].h.rx_graph[0][0];
            *tx_start = &intf[i].h.tx_graph[0][0];
            break;
    }

    switch (m_unit)
    {
        case byte:
            *rx_unit_y = *tx_unit_y = "B ";
            break;

        case kilobyte:
            *rx_unit_y = *tx_unit_y = "KB";
            for (w=0; w < 6; w++) rx_scale[w] /= 1024;
            for (w=0; w < 6; w++) tx_scale[w] /= 1024;
            break;
            
        case megabyte:
            *rx_unit_y = *tx_unit_y = "MB";
            for (w=0; w < 6; w++) rx_scale[w] /= 1048576;
            for (w=0; w < 6; w++) tx_scale[w] /= 1048576;
            break;

        case gigabyte:
            *rx_unit_y = *tx_unit_y = "GB";
            for (w=0; w < 6; w++) rx_scale[w] /= 1073741824;
            for (w=0; w < 6; w++) tx_scale[w] /= 1073741824;
            break;

        case terabyte:
            *rx_unit_y = *tx_unit_y = "TB";
            for (w=0; w < 6; w++) rx_scale[w] /= 1099511627776;
            for (w=0; w < 6; w++) tx_scale[w] /= 1099511627776;
            break;

        case dyanmic:
            if (rx_scale[4] >= 1073741824)
            {
                for (w=0; w < 6; w++) rx_scale[w] /= 1073741824;
                *rx_unit_y = "GB";
            }
            else if (rx_scale[4] >= 1048576)
            {
                for (w=0; w < 6; w++) rx_scale[w] /= 1048576;
                *rx_unit_y = "MB";
            }
            else if (rx_scale[4] >= 1024)
            {
                for (w=0; w < 6; w++) rx_scale[w] /= 1024;
                *rx_unit_y = "KB";
            }
            else
                *rx_unit_y = "B ";

            if (tx_scale[4] >= 1073741824)
            {
                for (w=0; w < 6; w++) tx_scale[w] /= 1073741824;
                *tx_unit_y = "GB";
            }
            else if (tx_scale[4] >= 1048576)
            {
                for (w=0; w < 6; w++) tx_scale[w] /= 1048576;
                *tx_unit_y = "MB";
            }
            else if (tx_scale[4] >= 1024)
            {
                for (w=0; w < 6; w++) tx_scale[w] /= 1024;
                *tx_unit_y = "KB";
            }
            else
                *tx_unit_y = "B ";
            break;
    }

}
